# coding: utf-8
# author: gabriel couture
from typing import List, Dict

from pyorthanc.datastructure.tree.series import Series
from pyorthanc.orthanc import Orthanc


class Study:
    """Represent an series that is in an Orthanc server

    This object has many getters that allow the user to retrieve metadata
    or the entire DICOM file of the Series
    """

    def __init__(
            self, study_identifier: str,
            orthanc: Orthanc,
            study_information: Dict = None) -> None:
        """Constructor

        Parameters
        ----------
        study_identifier
            Orthanc study identifier.
        orthanc
            Orthanc object.
        study_information
            Dictionary of study's information.
        """
        self.orthanc = orthanc

        self.study_identifier = study_identifier
        self.study_information = study_information

        self.series: List[Series] = []

    def get_identifier(self) -> str:
        """Get Study identifier

        Returns
        -------
        str
            Study identifier
        """
        return self.study_identifier

    def get_main_information(self) -> Dict:
        """Get Study information

        Returns
        -------
        Dict
            Dictionary of study information
        """
        if self.study_information is None:
            self.study_information = self.orthanc.get_study_information(
                self.study_identifier)

        return self.study_information

    def get_date(self) -> str:
        """Get study date

        Returns
        -------
        str
            Study date
        """
        return self.get_main_information()['MainDicomTags']['StudyDate']

    def get_time(self) -> str:
        """Get Study time

        Returns
        -------
        str
            Study time
        """
        return self.get_main_information()['MainDicomTags']['StudyTime']

    def get_id(self) -> str:
        """Get Study ID

        Returns
        -------
        str
            Study ID
        """
        return self.get_main_information()['MainDicomTags']['StudyID']

    def get_description(self) -> str:
        """Get study description

        Returns
        -------
        str
            Study description
        """
        return self.get_main_information()['MainDicomTags']['StudyDescription']

    def get_series(self) -> List[Series]:
        """Get Study series

        Returns
        -------
        List[Series]
            List of study's Series
        """
        return list(self.series)

    def build_series(self) -> None:
        """Build a list of the study's series
        """
        series_identifiers = self.orthanc.get_study_series_information(
            self.study_identifier)

        self.series = list(map(
            lambda i: Series(i['ID'], self.orthanc),
            series_identifiers
        ))

    def __str__(self):
        return f'Study (id={self.get_id()}, identifier={self.get_identifier()})'

    def trim(self) -> None:
        """Delete empty series
        """
        self.series = list(filter(
            lambda series: not series.is_empty(),
            self.series
        ))

    def is_empty(self) -> bool:
        """Check if series is empty

        Returns
        -------
        bool
            True if study has no instance
        """
        return self.series == []



# coding: utf-8
# author: gabriel couture
import unittest

import requests

from pyorthanc import Orthanc
from tests.integration import setup_server


class TestOrthancInstance(unittest.TestCase):
    A_INSTANCE_IDENTIFIER = '22dcf059-8fd3ade7-efb39ca3-7f46b248-0200abc9'

    A_LIST_OF_INSTANCE_IDENTIFIERS = [
        '22dcf059-8fd3ade7-efb39ca3-7f46b248-0200abc9',
        '348befe7-5be5ff53-70120381-3baa0cc2-e4e04220',
        'da2024f5-606f9e83-41b012bb-9dced1ea-77bcd599',
    ]

    @classmethod
    def setUpClass(cls) -> None:
        global orthanc_subprocess
        orthanc_subprocess = setup_server.setup_orthanc_server()

    @classmethod
    def tearDownClass(cls) -> None:
        global orthanc_subprocess
        setup_server.stop_orthanc_server_and_remove_data_directory(orthanc_subprocess)
        del orthanc_subprocess

    def setUp(self) -> None:
        self.orthanc = Orthanc(setup_server.ORTHANC_URL)

    def tearDown(self) -> None:
        self.orthanc = None
        setup_server.clear_data()

    def given_instances_in_orthanc_server(self):
        setup_server.setup_data()

    def test_givenOrthancWithData_whenGettingInstances_thenResultIsAListOfInstanceIdentifiers(self):
        self.given_instances_in_orthanc_server()

        result = self.orthanc.get_instances()

        self.assertIsInstance(result, list)
        self.assertNotEqual(len(result), 0)
        for instance_identifier in result:
            self.assertIn(instance_identifier, TestOrthancInstance.A_LIST_OF_INSTANCE_IDENTIFIERS)

    def test_givenOrthancWithoutData_whenGettingInstances_thenResultIsAnEmptyList(self):
        result = self.orthanc.get_instances()

        self.assertIsInstance(result, list)
        self.assertEqual(len(result), 0)

    def test_givenOrthancWithData_whenGettingInstanceFirstLevelTags_thenResultIsAListOfFirstLevelDICOMTags(self):
        self.given_instances_in_orthanc_server()

        result = self.orthanc.get_instance_first_level_tags(TestOrthancInstance.A_INSTANCE_IDENTIFIER)

        self.assertIsInstance(result, list)
        self.assertNotEqual(len(result), 0)
        for tag in result:
            tag_numbers = tag.split('-')
            self.assertEqual(len(tag_numbers), 2)

            for tag_number in tag_numbers:
                hex(int(tag_number, 16))

    def test_givenOrthancWithoutData_whenGettingInstanceFirstLevelTags_thenRaiseHTTPError(self):
        self.assertRaises(
            requests.exceptions.HTTPError,
            lambda: self.orthanc.get_instance_first_level_tags(TestOrthancInstance.A_INSTANCE_IDENTIFIER)
        )
